
/* ===========================================================================
 * Modulo:  matriz.c
 *
 * Version: 1.0 (23/11/2017)
 *
 * Autor:   Ivan Cantador Gutierrez, ivan.cantador@uam.es
 *          Departamento de Ingenieria Informatica
 *          Escuela Politecnica Superior - Universidad Autonoma de Madrid
 * =========================================================================== */

 
#include <stdio.h>
#include <stdlib.h>
#include "matriz.h"


/* ----------------------- */
/* Definicion de funciones */
/* ----------------------- */

/*****************************************************************************
 * Funcion: matriz_crear
 * 
 * Descripcion: 
 *    Crea una matriz de dimension definida por los argumentos de entrada,
 *    e inicializa sus elementos a 0.
 *
 * Argumentos de entrada:
 *    - numeroFilas
 *         numero de filas de la matriz a crear
 *    - numeroColumnas
 *         numero de columnas de la matriz a crear
 *
 * Salida:
 *    - puntero a la matriz crada, NULL en caso de error
 *****************************************************************************/

Matriz *matriz_crear( int numeroFilas, int numeroColumnas )
{
	Matriz	*matriz = NULL;
	int		i;

	if( numeroFilas<=0 || numeroColumnas<=0 ) 
	{
		return NULL;		
	}

	matriz = (Matriz *) malloc(sizeof(Matriz));
	if( !matriz ) 
	{
		return NULL;		
	}

	matriz->elementos = (double **) malloc(numeroFilas*sizeof(double *));
	if( !matriz->elementos ) 
	{
		free(matriz);
		return NULL;
	}

	for( i=0; i<numeroFilas; i++ ) 
	{
		matriz->elementos[i] = (double *) calloc(numeroColumnas, sizeof(double));
		if( !matriz->elementos[i] )
		{
			for( ; i>0; i-- ) 
			{
				free(matriz->elementos[i-1]);				
			}
			free(matriz->elementos);
			free(matriz);
			return NULL;
		}
	}

	matriz->numeroFilas = numeroFilas;
	matriz->numeroColumnas = numeroColumnas;
	
	return matriz;
}


/*****************************************************************************
 * Funcion: matriz_inicializar
 * 
 * Descripcion: 
 *    Inicializa el contenido de una matriz con unos valores dados.
 *
 * Argumentos de entrada:
 *    - matriz
 *         matriz a inicializar
 *    - valores
 *         array con los valores a copiar en la matriz
 *
 * Salida:
 *    - 0 si la ejecucion se realizo con exito, -1 en caso contrario
 *****************************************************************************/

int matriz_inicializar( Matriz *matriz, double **valores )
{
	int i, j;

	if( !matriz || !matriz->elementos || matriz->numeroFilas<=0 || matriz->numeroColumnas<=0 || !valores ) 
	{
		return -1;		
	}

	for( i=0; i<matriz->numeroFilas; i++ )
	{
		if( !matriz->elementos[i] || !valores[i] ) {
			return -1;			
		}
		
		for( j=0; j<matriz->numeroColumnas; j++ ) 
		{
			matriz->elementos[i][j] = valores[i][j];			
		}
	}

	return 0;
}


/*****************************************************************************
 * Funcion: matriz_visualizar
 * 
 * Descripcion: 
 *    Escribe en un fichero dado el contenido de una matriz.
 *
 * Argumentos de entrada:
 *    - matriz
 *         matriz a visualizar
 *    - fichero
 *         fichero donde escribir el contenido de la matriz
 *
 * Salida:
 *    - 0 si la ejecucion se realizo con exito, -1 en caso contrario
 *****************************************************************************/

int	matriz_visualizar( Matriz *matriz, FILE *fichero )
{
	int i, j;

	if( !matriz || !matriz->elementos || matriz->numeroFilas<=0 || matriz->numeroColumnas<=0 || !fichero ) 
	{
		return -1;
	}

	for( i=0; i<matriz->numeroFilas; i++ )
	{
		if( !matriz->elementos[i] ) {
			return -1;			
		}
		
		for( j=0; j<matriz->numeroColumnas; j++ ) {
			fprintf(fichero, "%f\t", matriz->elementos[i][j]);			
		}
		
		fprintf(fichero, "\n");
	}

	return 0;	
}


/*****************************************************************************
 * Funcion: matriz_liberar
 * 
 * Descripcion: 
 *    Libera la memoria reservada para una matriz dada.
 *
 * Argumentos de entrada:
 *    - matriz
 *         matriz a liberar
 *
 * Salida:
 *    - (ninguna)
 *****************************************************************************/

void matriz_liberar( Matriz *matriz )
{
	int i;

	if( matriz )
	{
		if( matriz->elementos )
		{
			for( i=0; i<matriz->numeroFilas; i++ ) 
			{
				if( matriz->elementos[i] ) 
				{
					free(matriz->elementos[i]);
					
				}				
			}
			free(matriz->elementos);
		}
		free(matriz);
	}
}
